#ifndef __ANYLEAF_H__
#define __ANYLEAF_H__

#if ARDUINO >= 100
#include "Arduino.h"
#else
#include "WProgram.h"
#endif

#include <Adafruit_ADS1015.h>
#include <SimpleKalmanFilter.h>
// #include <tuple>
// #include "optional.hpp"
// using nonstd::nullopt;

struct Twople {
    public:
        Twople();
        Twople(float a, float b);
        float a;
        float b;
};

struct CalPt {
public:
    CalPt();
    CalPt(float V_, float pH_, float T_);
    float V; // voltage, in Volts
    float pH;
    float T; // in Celsius
};

enum class CalSlot {
    // Keeps our calibration organized, so we track when to overwrite.
    One,
    Two,
    Three,
};

class PhSensor {
public:
    PhSensor();

    SimpleKalmanFilter filter;
    // void predict();
    // void update();
    float read();
    float read(float t);
    float read_raw();
    float read_raw(float t);
    float read_voltage();
    float read_temp();
    Twople calibrate(CalSlot slot, float pH);
    Twople calibrate(CalSlot slot, float pH, float t);
    void calibrate_all(CalPt pt0, CalPt pt1);
    void calibrate_all(CalPt pt0, CalPt pt1, CalPt pt2);
    void reset_calibration();

private:
    Adafruit_ADS1115 adc;
    float last_meas;
    CalPt cal_1;
    CalPt cal_2;
    CalPt cal_3;
};

float voltage_from_adc(int16_t digi);
float lg(float x0, float y0, float x1, float y1, float x2, float y2, float X);
float ph_from_voltage(float V, float temp, CalPt cal_0, CalPt cal_1);
float ph_from_voltage(float V, float temp, CalPt cal_0, CalPt cal_1, CalPt cal_2);
// float ph_from_voltage(float V, float temp, CalPt cal_0, CalPt cal_1, optional<CapPt> cal_2)
float temp_from_voltage(float V);
#endif